#include <stdio.h>
#include <stdlib.h>

#ifndef SIZE
#define SIZE 10
#endif

#define ERROR_FILE_FAIL 100
#define ERROR_MALLOC_FAIL 101
#define ERROR_REALLOC_FAIL 102

int main(int argc, char *argv[])
{
    const char *fname = argc > 1 ? argv[1] : NULL;
    FILE *fd = fopen(fname, "r");
    if (fd == NULL) {
        fprintf(stderr, "Cannot open file %s\n", fname);
        return ERROR_FILE_FAIL;
    }
    char *line = malloc(SIZE + 1);
    if (!line) {
        fprintf(stderr, "Cannot allocate\n");
        return ERROR_MALLOC_FAIL;
    }
    int capacity = SIZE + 1;
    int len = 0;
    char r;
    while ((r = getc(fd)) != EOF && r != '\n') {
        if (len == capacity) {
            char *t = realloc(line, capacity + SIZE + 1);
            if (t == NULL) {
                fprintf(stderr, "Cannot reallocate!\n");
                free(line);
                fclose(fd);
                return ERROR_REALLOC_FAIL;
            }
            line = t;
            capacity += SIZE;
        }
        line[len++] = r;
    }
    line[len] = '\0'; // we need to ensure string is terminated
    printf("Readline \"%s\"\n", line);
    free(line);
    return EXIT_SUCCESS;
}
